@@ -19,8 +19,8 @@
  *
  * {1.34} - 19980630 - Arnaldo Carvalho de Melo <acme@conectiva.com.br>
  *                     - gettext instead of catgets for i18n
- *          10/1998  - Andi Kleen. Use interface list primitives.       
- *	    20001008 - Bernd Eckenfels, Patch from RH for setting mtu 
+ *          10/1998  - Andi Kleen. Use interface list primitives.
+ *	    20001008 - Bernd Eckenfels, Patch from RH for setting mtu
  *			(default AF was wrong)
  *          20010404 - Arnaldo Carvalho de Melo, use setlocale
  */
@@ -85,7 +85,7 @@ struct in6_ifreq {
 #include "sockets.h"
 #include "util.h"
 
-char *Release = RELEASE, *Version = "ifconfig 1.42 (2001-04-13)";
+static char *Release = RELEASE;
 
 int opt_a = 0;			/* show all interfaces          */
 int opt_v = 0;			/* debugging output flag        */
@@ -93,10 +93,8 @@ int opt_v = 0;			/* debugging output flag        */
 int addr_family = 0;		/* currently selected AF        */
 
 /* for ipv4 add/del modes */
-static int get_nmbc_parent(char *parent, unsigned long *nm, 
-			   unsigned long *bc);
-static int set_ifstate(char *parent, unsigned long ip,
-		       unsigned long nm, unsigned long bc,
+static int get_nmbc_parent(char *parent, in_addr_t *nm, in_addr_t *bc);
+static int set_ifstate(char *parent, in_addr_t ip, in_addr_t nm, in_addr_t bc,
 		       int flag);
 
 static int if_print(char *ifname)
@@ -115,11 +113,11 @@ static int if_print(char *ifname)
 	if (!ife) {
 		return -1;
 	}
-	res = do_if_fetch(ife); 
-	if (res >= 0) 
+	res = do_if_fetch(ife);
+	if (res >= 0)
 	    ife_print(ife);
     }
-    return res; 
+    return res;
 }
 
 /* Set a certain interface flag. */
@@ -129,7 +127,7 @@ static int set_flag(char *ifname, short flag)
 
     safe_strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
     if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, _("%s: ERROR while getting interface flags: %s\n"), 
+	fprintf(stderr, _("%s: ERROR while getting interface flags: %s\n"),
 		ifname,	strerror(errno));
 	return (-1);
     }
@@ -161,7 +159,7 @@ static int clr_flag(char *ifname, short flag)
 
     safe_strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
     if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, _("%s: ERROR while getting interface flags: %s\n"), 
+	fprintf(stderr, _("%s: ERROR while getting interface flags: %s\n"),
 		ifname, strerror(errno));
 	return -1;
     }
@@ -193,58 +191,61 @@ static int test_flag(char *ifname, short flags)
 
     safe_strncpy(ifr.ifr_name, ifname, IFNAMSIZ);
     if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) {
-	fprintf(stderr, _("%s: ERROR while testing interface flags: %s\n"), 
+	fprintf(stderr, _("%s: ERROR while testing interface flags: %s\n"),
 		ifname, strerror(errno));
 	return -1;
     }
     return (ifr.ifr_flags & flags);
 }
 
-static void usage(void)
+static void usage(int rc)
 {
-    fprintf(stderr, _("Usage:\n  ifconfig [-a] [-v] [-s] <interface> [[<AF>] <address>]\n"));
+    FILE *fp = rc ? stderr : stdout;
+    fprintf(fp, _("Usage:\n  ifconfig [-a] [-v] [-s] <interface> [[<AF>] <address>]\n"));
 #if HAVE_AFINET
-    fprintf(stderr, _("  [add <address>[/<prefixlen>]]\n"));
-    fprintf(stderr, _("  [del <address>[/<prefixlen>]]\n"));
-    fprintf(stderr, _("  [[-]broadcast [<address>]]  [[-]pointopoint [<address>]]\n"));
-    fprintf(stderr, _("  [netmask <address>]  [dstaddr <address>]  [tunnel <address>]\n"));
+    fprintf(fp, _("  [add <address>[/<prefixlen>]]\n"));
+    fprintf(fp, _("  [del <address>[/<prefixlen>]]\n"));
+    fprintf(fp, _("  [[-]broadcast [<address>]]  [[-]pointopoint [<address>]]\n"));
+    fprintf(fp, _("  [netmask <address>]  [dstaddr <address>]  [tunnel <address>]\n"));
 #endif
 #ifdef SIOCSKEEPALIVE
-    fprintf(stderr, _("  [outfill <NN>] [keepalive <NN>]\n"));
+    fprintf(fp, _("  [outfill <NN>] [keepalive <NN>]\n"));
 #endif
-    fprintf(stderr, _("  [hw <HW> <address>]  [mtu <NN>]\n"));
-    fprintf(stderr, _("  [[-]trailers]  [[-]arp]  [[-]allmulti]\n"));
-    fprintf(stderr, _("  [multicast]  [[-]promisc]\n"));
-    fprintf(stderr, _("  [mem_start <NN>]  [io_addr <NN>]  [irq <NN>]  [media <type>]\n"));
+    fprintf(fp, _("  [hw <HW> <address>]  [mtu <NN>]\n"));
+    fprintf(fp, _("  [[-]trailers]  [[-]arp]  [[-]allmulti]\n"));
+    fprintf(fp, _("  [multicast]  [[-]promisc]\n"));
+    fprintf(fp, _("  [mem_start <NN>]  [io_addr <NN>]  [irq <NN>]  [media <type>]\n"));
 #ifdef HAVE_TXQUEUELEN
-    fprintf(stderr, _("  [txqueuelen <NN>]\n"));
+    fprintf(fp, _("  [txqueuelen <NN>]\n"));
+#endif
+#ifdef SIOCSIFNAME
+    fprintf(fp, _("  [name <newname>]\n"));
 #endif
 #ifdef HAVE_DYNAMIC
-    fprintf(stderr, _("  [[-]dynamic]\n"));
+    fprintf(fp, _("  [[-]dynamic]\n"));
 #endif
-    fprintf(stderr, _("  [up|down] ...\n\n"));
+    fprintf(fp, _("  [up|down] ...\n\n"));
 
-    fprintf(stderr, _("  <HW>=Hardware Type.\n"));
-    fprintf(stderr, _("  List of possible hardware types:\n"));
+    fprintf(fp, _("  <HW>=Hardware Type.\n"));
+    fprintf(fp, _("  List of possible hardware types:\n"));
     print_hwlist(0); /* 1 = ARPable */
-    fprintf(stderr, _("  <AF>=Address family. Default: %s\n"), DFLT_AF);
-    fprintf(stderr, _("  List of possible address families:\n"));
+    fprintf(fp, _("  <AF>=Address family. Default: %s\n"), DFLT_AF);
+    fprintf(fp, _("  List of possible address families:\n"));
     print_aflist(0); /* 1 = routeable */
-    exit(E_USAGE);
+    exit(rc);
 }
 
 static void version(void)
 {
-    fprintf(stderr, "%s\n%s\n", Release, Version);
-    exit(E_USAGE);
+    printf("%s\n", Release);
+    exit(E_VERSION);
 }
 
 static int set_netmask(int skfd, struct ifreq *ifr, struct sockaddr *sa)
 {
     int err = 0;
 
-    memcpy((char *) &ifr->ifr_netmask, (char *) sa,
-	   sizeof(struct sockaddr));
+    memcpy(&ifr->ifr_netmask, sa, sizeof(struct sockaddr));
     if (ioctl(skfd, SIOCSIFNETMASK, ifr) < 0) {
 	fprintf(stderr, "SIOCSIFNETMASK: %s\n",
 		strerror(errno));
@@ -255,19 +256,20 @@ static int set_netmask(int skfd, struct ifreq *ifr, struct sockaddr *sa)
 
 int main(int argc, char **argv)
 {
-    struct sockaddr sa;
-    struct sockaddr samask;
-    struct sockaddr_in sin;
+    struct sockaddr_storage _sa, _samask;
+    struct sockaddr *sa = (struct sockaddr *)&_sa;
+    struct sockaddr *samask = (struct sockaddr *)&_samask;
+    struct sockaddr_in *sin = (struct sockaddr_in *)&_sa;
     char host[128];
-    struct aftype *ap;
-    struct hwtype *hw;
+    const struct aftype *ap;
+    const struct hwtype *hw;
     struct ifreq ifr;
     int goterr = 0, didnetmask = 0, neednetmask=0;
     char **spp;
     int fd;
 #if HAVE_AFINET6
     extern struct aftype inet6_aftype;
-    struct sockaddr_in6 sa6;
+    struct sockaddr_in6 *sin6 = (struct sockaddr_in6 *)&_sa;
     struct in6_ifreq ifr6;
     unsigned long prefix_len;
     char *cp;
@@ -277,7 +279,7 @@ int main(int argc, char **argv)
 #endif
 
 #if I18N
-    setlocale (LC_ALL, "");
+    setlocale(LC_ALL, "");
     bindtextdomain("net-tools", "/usr/share/locale");
     textdomain("net-tools");
 #endif
@@ -294,17 +296,17 @@ int main(int argc, char **argv)
 
 	else if (!strcmp(*argv, "-v"))
 	    opt_v = 1;
-	
+
 	else if (!strcmp(*argv, "-V") || !strcmp(*argv, "-version") ||
 	    !strcmp(*argv, "--version"))
 	    version();
 
 	else if (!strcmp(*argv, "-?") || !strcmp(*argv, "-h") ||
 	    !strcmp(*argv, "-help") || !strcmp(*argv, "--help"))
-	    usage();
+	    usage(E_USAGE);
 
 	else {
-	    fprintf(stderr, _("ifconfig: option `%s' not recognised.\n"), 
+	    fprintf(stderr, _("ifconfig: option `%s' not recognised.\n"),
 		    argv[0]);
 	    fprintf(stderr, _("ifconfig: `--help' gives usage information.\n"));
 	    exit(1);
@@ -338,9 +340,9 @@ int main(int argc, char **argv)
     /* The next argument is either an address family name, or an option. */
     if ((ap = get_aftype(*spp)) != NULL)
 	spp++; /* it was a AF name */
-    else 
+    else
 	ap = get_aftype(DFLT_AF);
-	
+
     if (ap) {
 	addr_family = ap->af;
 	skfd = ap->fd;
@@ -361,7 +363,7 @@ int main(int argc, char **argv)
 #ifdef IFF_PORTSEL
 	if (!strcmp(*spp, "media") || !strcmp(*spp, "port")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if (!strcasecmp(*spp, "auto")) {
 		goterr |= set_flag(ifr.ifr_name, IFF_AUTOMEDIA);
 	    } else {
@@ -385,7 +387,7 @@ int main(int argc, char **argv)
 		    goterr = 1;
 		} else {
 		    if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
-			perror("port: SIOCGIFMAP"); 
+			perror("port: SIOCGIFMAP");
 			goterr = 1;
 			continue;
 		    }
@@ -473,7 +475,7 @@ int main(int argc, char **argv)
 
 	if (!strcmp(*spp, "mtu")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    ifr.ifr_mtu = atoi(*spp);
 	    if (ioctl(skfd, SIOCSIFMTU, &ifr) < 0) {
 		fprintf(stderr, "SIOCSIFMTU: %s\n", strerror(errno));
@@ -485,7 +487,7 @@ int main(int argc, char **argv)
 #ifdef SIOCSKEEPALIVE
 	if (!strcmp(*spp, "keepalive")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    ifr.ifr_data = (caddr_t) (uintptr_t) atoi(*spp);
 	    if (ioctl(skfd, SIOCSKEEPALIVE, &ifr) < 0) {
 		fprintf(stderr, "SIOCSKEEPALIVE: %s\n", strerror(errno));
@@ -499,7 +501,7 @@ int main(int argc, char **argv)
 #ifdef SIOCSOUTFILL
 	if (!strcmp(*spp, "outfill")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    ifr.ifr_data = (caddr_t) (uintptr_t) atoi(*spp);
 	    if (ioctl(skfd, SIOCSOUTFILL, &ifr) < 0) {
 		fprintf(stderr, "SIOCSOUTFILL: %s\n", strerror(errno));
@@ -520,7 +522,7 @@ int main(int argc, char **argv)
 	if (!strcmp(*spp, "broadcast")) {
 	    if (*++spp != NULL) {
 		safe_strncpy(host, *spp, (sizeof host));
-		if (ap->input(0, host, &sa) < 0) {
+		if (ap->input(0, host, &_sa) < 0) {
 		    if (ap->herror)
 		    	ap->herror(host);
 		    else
@@ -529,8 +531,7 @@ int main(int argc, char **argv)
 		    spp++;
 		    continue;
 		}
-		memcpy((char *) &ifr.ifr_broadaddr, (char *) &sa,
-		       sizeof(struct sockaddr));
+		memcpy(&ifr.ifr_broadaddr, sa, sizeof(struct sockaddr));
 		if (ioctl(ap->fd, SIOCSIFBRDADDR, &ifr) < 0) {
 		    fprintf(stderr, "SIOCSIFBRDADDR: %s\n",
 			    strerror(errno));
@@ -543,9 +544,9 @@ int main(int argc, char **argv)
 	}
 	if (!strcmp(*spp, "dstaddr")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    safe_strncpy(host, *spp, (sizeof host));
-	    if (ap->input(0, host, &sa) < 0) {
+	    if (ap->input(0, host, &_sa) < 0) {
 		    if (ap->herror)
 		    	ap->herror(host);
 		    else
@@ -554,8 +555,7 @@ int main(int argc, char **argv)
 		spp++;
 		continue;
 	    }
-	    memcpy((char *) &ifr.ifr_dstaddr, (char *) &sa,
-		   sizeof(struct sockaddr));
+	    memcpy(&ifr.ifr_dstaddr, sa, sizeof(struct sockaddr));
 	    if (ioctl(ap->fd, SIOCSIFDSTADDR, &ifr) < 0) {
 		fprintf(stderr, "SIOCSIFDSTADDR: %s\n",
 			strerror(errno));
@@ -566,9 +566,9 @@ int main(int argc, char **argv)
 	}
 	if (!strcmp(*spp, "netmask")) {
 	    if (*++spp == NULL || didnetmask)
-		usage();
+		usage(E_OPTERR);
 	    safe_strncpy(host, *spp, (sizeof host));
-	    if (ap->input(0, host, &sa) < 0) {
+	    if (ap->input(0, host, &_sa) < 0) {
 		    if (ap->herror)
 		    	ap->herror(host);
 		    else
@@ -578,14 +578,14 @@ int main(int argc, char **argv)
 		continue;
 	    }
 	    didnetmask++;
-	    goterr |= set_netmask(ap->fd, &ifr, &sa);
+	    goterr |= set_netmask(ap->fd, &ifr, sa);
 	    spp++;
 	    continue;
 	}
 #ifdef HAVE_TXQUEUELEN
 	if (!strcmp(*spp, "txqueuelen")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    ifr.ifr_qlen = strtoul(*spp, NULL, 0);
 	    if (ioctl(skfd, SIOCSIFTXQLEN, &ifr) < 0) {
 		fprintf(stderr, "SIOCSIFTXQLEN: %s\n", strerror(errno));
@@ -596,12 +596,26 @@ int main(int argc, char **argv)
 	}
 #endif
 
+#ifdef SIOCSIFNAME
+	if (!strcmp(*spp, "name")) {
+	    if (*++spp == NULL)
+		usage(E_OPTERR);
+	    safe_strncpy(ifr.ifr_newname, *spp, IFNAMSIZ);
+	    if (ioctl(skfd, SIOCSIFNAME, &ifr) < 0) {
+		fprintf(stderr, "SIOCSIFNAME: %s\n", strerror(errno));
+		goterr = 1;
+	    }
+	    spp++;
+	    continue;
+	}
+#endif
+
 	if (!strcmp(*spp, "mem_start")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
-		fprintf(stderr, "mem_start: SIOCGIFMAP: %s\n", strerror(errno)); 
-		spp++; 
+		fprintf(stderr, "mem_start: SIOCGIFMAP: %s\n", strerror(errno));
+		spp++;
 		goterr = 1;
 		continue;
 	    }
@@ -615,10 +629,10 @@ int main(int argc, char **argv)
 	}
 	if (!strcmp(*spp, "io_addr")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
-		fprintf(stderr, "io_addr: SIOCGIFMAP: %s\n", strerror(errno)); 
-		spp++; 
+		fprintf(stderr, "io_addr: SIOCGIFMAP: %s\n", strerror(errno));
+		spp++;
 		goterr = 1;
 		continue;
 	    }
@@ -632,11 +646,11 @@ int main(int argc, char **argv)
 	}
 	if (!strcmp(*spp, "irq")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if (ioctl(skfd, SIOCGIFMAP, &ifr) < 0) {
-		fprintf(stderr, "irq: SIOCGIFMAP: %s\n", strerror(errno)); 
+		fprintf(stderr, "irq: SIOCGIFMAP: %s\n", strerror(errno));
 		goterr = 1;
-		spp++; 
+		spp++;
 		continue;
 	    }
 	    ifr.ifr_map.irq = atoi(*spp);
@@ -647,18 +661,18 @@ int main(int argc, char **argv)
 	    spp++;
 	    continue;
 	}
-	if (!strcmp(*spp, "-pointopoint")) {
+	if (!strcmp(*spp, "-pointopoint") || !strcmp(*spp, "-pointtopoint")) {
 	    goterr |= clr_flag(ifr.ifr_name, IFF_POINTOPOINT);
 	    spp++;
 	    if (test_flag(ifr.ifr_name, IFF_POINTOPOINT) > 0)
 	    	fprintf(stderr, _("Warning: Interface %s still in POINTOPOINT mode.\n"), ifr.ifr_name);
 	    continue;
 	}
-	if (!strcmp(*spp, "pointopoint")) {
+	if (!strcmp(*spp, "pointopoint") || !strcmp(*spp, "pointtopoint")) {
 	    if (*(spp + 1) != NULL) {
 		spp++;
 		safe_strncpy(host, *spp, (sizeof host));
-		if (ap->input(0, host, &sa)) {
+		if (ap->input(0, host, &_sa)) {
 		    if (ap->herror)
 		    	ap->herror(host);
 		    else
@@ -667,8 +681,7 @@ int main(int argc, char **argv)
 		    spp++;
 		    continue;
 		}
-		memcpy((char *) &ifr.ifr_dstaddr, (char *) &sa,
-		       sizeof(struct sockaddr));
+		memcpy(&ifr.ifr_dstaddr, sa, sizeof(struct sockaddr));
 		if (ioctl(ap->fd, SIOCSIFDSTADDR, &ifr) < 0) {
 		    fprintf(stderr, "SIOCSIFDSTADDR: %s\n",
 			    strerror(errno));
@@ -682,9 +695,9 @@ int main(int argc, char **argv)
 
 	if (!strcmp(*spp, "hw")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if ((hw = get_hwtype(*spp)) == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if (hw->input == NULL) {
 	    	fprintf(stderr, _("hw address type `%s' has no handler to set address. failed.\n"), *spp);
 	    	spp+=2;
@@ -692,16 +705,15 @@ int main(int argc, char **argv)
 	    	continue;
 	    }
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    safe_strncpy(host, *spp, (sizeof host));
-	    if (hw->input(host, &sa) < 0) {
+	    if (hw->input(host, &_sa) < 0) {
 		fprintf(stderr, _("%s: invalid %s address.\n"), host, hw->name);
 		goterr = 1;
 		spp++;
 		continue;
 	    }
-	    memcpy((char *) &ifr.ifr_hwaddr, (char *) &sa,
-		   sizeof(struct sockaddr));
+	    memcpy(&ifr.ifr_hwaddr, sa, sizeof(struct sockaddr));
 	    if (ioctl(skfd, SIOCSIFHWADDR, &ifr) < 0) {
 		if (errno == EBUSY)
 			fprintf(stderr, "SIOCSIFHWADDR: %s - you may need to down the interface\n",
@@ -717,21 +729,20 @@ int main(int argc, char **argv)
 #if HAVE_AFINET || HAVE_AFINET6
 	if (!strcmp(*spp, "add")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 #if HAVE_AFINET6
 	    if (strchr(*spp, ':')) {
 		/* INET6 */
 		if ((cp = strchr(*spp, '/'))) {
 		    prefix_len = atol(cp + 1);
 		    if ((prefix_len < 0) || (prefix_len > 128))
-			usage();
+			usage(E_OPTERR);
 		    *cp = 0;
 		} else {
 		    prefix_len = 128;
 		}
 		safe_strncpy(host, *spp, (sizeof host));
-		if (inet6_aftype.input(1, host, 
-				       (struct sockaddr *) &sa6) < 0) {
+		if (inet6_aftype.input(1, host, &_sa) < 0) {
 		    if (inet6_aftype.herror)
 		    	inet6_aftype.herror(host);
 		    else
@@ -740,12 +751,11 @@ int main(int argc, char **argv)
 		    spp++;
 		    continue;
 		}
-		memcpy((char *) &ifr6.ifr6_addr, (char *) &sa6.sin6_addr,
-		       sizeof(struct in6_addr));
+		memcpy(&ifr6.ifr6_addr, &sin6->sin6_addr, sizeof(struct in6_addr));
 
 		fd = get_socket_for_af(AF_INET6);
 		if (fd < 0) {
-		    fprintf(stderr, 
+		    fprintf(stderr,
 			    _("No support for INET6 on this system.\n"));
 		    goterr = 1;
 		    spp++;
@@ -769,9 +779,9 @@ int main(int argc, char **argv)
 #endif
 #if HAVE_AFINET
 	    { /* ipv4 address a.b.c.d */
-		unsigned long ip, nm, bc;
+		in_addr_t ip, nm, bc;
 		safe_strncpy(host, *spp, (sizeof host));
-		if (inet_aftype.input(0, host, (struct sockaddr *)&sin) < 0) {
+		if (inet_aftype.input(0, host, &_sa) < 0) {
 		    ap->herror(host);
 		    goterr = 1;
 		    spp++;
@@ -779,15 +789,15 @@ int main(int argc, char **argv)
 		}
 		fd = get_socket_for_af(AF_INET);
 		if (fd < 0) {
-		    fprintf(stderr, 
+		    fprintf(stderr,
 			    _("No support for INET on this system.\n"));
 		    goterr = 1;
 		    spp++;
 		    continue;
 		}
 
-		memcpy(&ip, &sin.sin_addr.s_addr, sizeof(unsigned long));
-		
+		memcpy(&ip, &sin->sin_addr.s_addr, sizeof(ip));
+
 		if (get_nmbc_parent(ifr.ifr_name, &nm, &bc) < 0) {
 			fprintf(stderr, _("Interface %s not initialized\n"),
 				ifr.ifr_name);
@@ -796,7 +806,7 @@ int main(int argc, char **argv)
 			continue;
 		}
 		set_ifstate(ifr.ifr_name, ip, nm, bc, 1);
-		
+
 	    }
 	    spp++;
 	    continue;
@@ -809,7 +819,7 @@ int main(int argc, char **argv)
 #if HAVE_AFINET || HAVE_AFINET6
 	if (!strcmp(*spp, "del")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 
 #ifdef SIOCDIFADDR
 #if HAVE_AFINET6
@@ -817,25 +827,24 @@ int main(int argc, char **argv)
 		if ((cp = strchr(*spp, '/'))) {
 		    prefix_len = atol(cp + 1);
 		    if ((prefix_len < 0) || (prefix_len > 128))
-			usage();
+			usage(E_OPTERR);
 		    *cp = 0;
 		} else {
 		    prefix_len = 128;
 		}
 		safe_strncpy(host, *spp, (sizeof host));
-		if (inet6_aftype.input(1, host, 
-				       (struct sockaddr *) &sa6) < 0) {
+		if (inet6_aftype.input(1, host, &_sa) < 0) {
 		    inet6_aftype.herror(host);
 		    goterr = 1;
 		    spp++;
 		    continue;
 		}
-		memcpy((char *) &ifr6.ifr6_addr, (char *) &sa6.sin6_addr,
+		memcpy(&ifr6.ifr6_addr, &sin6->sin6_addr,
 		       sizeof(struct in6_addr));
-		
+
 		fd = get_socket_for_af(AF_INET6);
 		if (fd < 0) {
-		    fprintf(stderr, 
+		    fprintf(stderr,
 			    _("No support for INET6 on this system.\n"));
 		    goterr = 1;
 		    spp++;
@@ -863,9 +872,9 @@ int main(int argc, char **argv)
 #if HAVE_AFINET
 	    {
 		/* ipv4 address a.b.c.d */
-		unsigned long ip, nm, bc;
+		in_addr_t ip, nm, bc;
 		safe_strncpy(host, *spp, (sizeof host));
-		if (inet_aftype.input(0, host, (struct sockaddr *)&sin) < 0) {
+		if (inet_aftype.input(0, host, &_sa) < 0) {
 		    ap->herror(host);
 		    goterr = 1;
 		    spp++;
@@ -878,11 +887,11 @@ int main(int argc, char **argv)
 		    spp++;
 		    continue;
 		}
-		
+
 		/* Clear "ip" in case sizeof(unsigned long) > sizeof(sin.sin_addr.s_addr) */
 		ip = 0;
-		memcpy(&ip, &sin.sin_addr.s_addr, sizeof(sin.sin_addr.s_addr));
-		
+		memcpy(&ip, &sin->sin_addr.s_addr, sizeof(ip));
+
 		if (get_nmbc_parent(ifr.ifr_name, &nm, &bc) < 0) {
 		    fprintf(stderr, _("Interface %s not initialized\n"),
 			    ifr.ifr_name);
@@ -905,24 +914,23 @@ int main(int argc, char **argv)
 #if HAVE_AFINET6
 	if (!strcmp(*spp, "tunnel")) {
 	    if (*++spp == NULL)
-		usage();
+		usage(E_OPTERR);
 	    if ((cp = strchr(*spp, '/'))) {
 		prefix_len = atol(cp + 1);
 		if ((prefix_len < 0) || (prefix_len > 128))
-		    usage();
+		    usage(E_OPTERR);
 		*cp = 0;
 	    } else {
 		prefix_len = 128;
 	    }
 	    safe_strncpy(host, *spp, (sizeof host));
-	    if (inet6_aftype.input(1, host, (struct sockaddr *) &sa6) < 0) {
+	    if (inet6_aftype.input(1, host, &_sa) < 0) {
 		inet6_aftype.herror(host);
 		goterr = 1;
 		spp++;
 		continue;
 	    }
-	    memcpy((char *) &ifr6.ifr6_addr, (char *) &sa6.sin6_addr,
-		   sizeof(struct in6_addr));
+	    memcpy(&ifr6.ifr6_addr, &sin6->sin6_addr, sizeof(struct in6_addr));
 
 	    fd = get_socket_for_af(AF_INET6);
 	    if (fd < 0) {
@@ -953,16 +961,16 @@ int main(int argc, char **argv)
 	/* If the next argument is a valid hostname, assume OK. */
 	safe_strncpy(host, *spp, (sizeof host));
 
-	/* FIXME: sa is too small for INET6 addresses, inet6 should use that too, 
+	/* FIXME: sa is too small for INET6 addresses, inet6 should use that too,
 	   broadcast is unexpected */
 	if (ap->getmask) {
-	    switch (ap->getmask(host, &samask, NULL)) {
+	    switch (ap->getmask(host, &_samask, NULL)) {
 	    case -1:
-		usage();
+		usage(E_OPTERR);
 		break;
 	    case 1:
 		if (didnetmask)
-		    usage();
+		    usage(E_OPTERR);
 
 		// remeber to set the netmask from samask later
 		neednetmask = 1;
@@ -973,14 +981,15 @@ int main(int argc, char **argv)
 	   fprintf(stderr, _("ifconfig: Cannot set address for this protocol family.\n"));
 	   exit(1);
 	}
-	if (ap->input(0, host, &sa) < 0) {
+	if (ap->input(0, host, &_sa) < 0) {
 	    if (ap->herror)
 	    	ap->herror(host);
 	    else
-	    	fprintf(stderr,_("ifconfig: error resolving '%s' to set address for af=%s\n"), host, ap->name); fprintf(stderr,
-	    _("ifconfig: `--help' gives usage information.\n")); exit(1);
+	    	fprintf(stderr,_("ifconfig: error resolving '%s' to set address for af=%s\n"), host, ap->name);
+	    fprintf(stderr, _("ifconfig: `--help' gives usage information.\n"));
+	    exit(1);
 	}
-	memcpy((char *) &ifr.ifr_addr, (char *) &sa, sizeof(struct sockaddr));
+	memcpy(&ifr.ifr_addr, sa, sizeof(struct sockaddr));
 	{
 	    int r = 0;		/* to shut gcc up */
 	    switch (ap->af) {
@@ -1018,17 +1027,27 @@ int main(int argc, char **argv)
        /*
         * Don't do the set_flag() if the address is an alias with a - at the
         * end, since it's deleted already! - Roman
+        * Same goes if they used address 0.0.0.0 as the kernel uses this to
+        * destroy aliases.
         *
         * Should really use regex.h here, not sure though how well it'll go
-        * with the cross-platform support etc. 
+        * with the cross-platform support etc.
         */
         {
             char *ptr;
             short int found_colon = 0;
+            short int bring_up = 1;
             for (ptr = ifr.ifr_name; *ptr; ptr++ )
                 if (*ptr == ':') found_colon++;
-                
-            if (!(found_colon && *(ptr - 1) == '-'))
+
+            if (found_colon) {
+                if (ptr[-1] == '-')
+                    bring_up = 0;
+                else if (ap->af == AF_INET && sin->sin_addr.s_addr == 0)
+                    bring_up = 0;
+            }
+
+            if (bring_up)
                 goterr |= set_flag(ifr.ifr_name, (IFF_UP | IFF_RUNNING));
         }
 
@@ -1036,7 +1055,7 @@ int main(int argc, char **argv)
     }
 
     if (neednetmask) {
-	goterr |= set_netmask(skfd, &ifr, &samask);
+	goterr |= set_netmask(skfd, &ifr, samask);
 	didnetmask++;
     }
 
@@ -1075,7 +1094,7 @@ static int do_ifcmd(struct interface *x, struct ifcmd *ptr)
     char *z, *e;
     struct sockaddr_in *sin;
     int i;
-    
+
     if (do_if_fetch(x) < 0)
 	return 0;
     if (strncmp(x->name, ptr->base, ptr->baselen) != 0)
@@ -1091,13 +1110,13 @@ static int do_ifcmd(struct interface *x, struct ifcmd *ptr)
     if (i < 0 || i > 255)
 	abort();
     searcher[i] = 1;
-    
+
     /* copy */
-    sin = (struct sockaddr_in *)&x->dstaddr;
+    sin = (struct sockaddr_in *)&x->dstaddr_sas;
     if (sin->sin_addr.s_addr != ptr->addr) {
 	return 0;
     }
-    
+
     if (ptr->flag) {
 	/* turn UP */
 	if (set_flag(x->name, IFF_UP | IFF_RUNNING) == -1)
@@ -1107,49 +1126,48 @@ static int do_ifcmd(struct interface *x, struct ifcmd *ptr)
 	if (clr_flag(x->name, IFF_UP) == -1)
 	    return -1;
     }
-    
+
     return 1; /* all done! */
 }
 
 
 static int get_nmbc_parent(char *parent,
-			   unsigned long *nm, unsigned long *bc)
+			   in_addr_t *nm, in_addr_t *bc)
 {
     struct interface *i;
     struct sockaddr_in *sin;
-    
+
     i = lookup_interface(parent);
     if (!i)
 	return -1;
     if (do_if_fetch(i) < 0)
 	return 0;
-    sin = (struct sockaddr_in *)&i->netmask;
-    memcpy(nm, &sin->sin_addr.s_addr, sizeof(unsigned long));
-    sin = (struct sockaddr_in *)&i->broadaddr;
-    memcpy(bc, &sin->sin_addr.s_addr, sizeof(unsigned long));
+    sin = (struct sockaddr_in *)&i->netmask_sas;
+    memcpy(nm, &sin->sin_addr.s_addr, sizeof(*nm));
+    sin = (struct sockaddr_in *)&i->broadaddr_sas;
+    memcpy(bc, &sin->sin_addr.s_addr, sizeof(*bc));
     return 0;
 }
 
-static int set_ifstate(char *parent, unsigned long ip,
-		       unsigned long nm, unsigned long bc,
+static int set_ifstate(char *parent, in_addr_t ip, in_addr_t nm, in_addr_t bc,
 		       int flag)
 {
     char buf[IFNAMSIZ];
     struct ifcmd pt;
     int i;
-    
+
     pt.base = parent;
     pt.baselen = strlen(parent);
     pt.addr = ip;
     pt.flag = flag;
     memset(searcher, 0, sizeof(searcher));
-    i = for_all_interfaces((int (*)(struct interface *,void *))do_ifcmd, 
+    i = for_all_interfaces((int (*)(struct interface *,void *))do_ifcmd,
 			   &pt);
     if (i == -1)
 	return -1;
     if (i == 1)
 	return 0;
-    
+
     /* add a new interface */
     for (i = 0; i < 256; i++)
 	if (searcher[i] == 0)
@@ -1157,7 +1175,7 @@ static int set_ifstate(char *parent, unsigned long ip,
 
     if (i == 256)
 	return -1; /* FAILURE!!! out of ip addresses */
-    
+
     if (snprintf(buf, IFNAMSIZ, "%s:%d", parent, i) > IFNAMSIZ)
 	return -1;
     if (set_ip_using(buf, SIOCSIFADDR, ip) == -1)
